@@ -1,7 +1,7 @@
 ###########################################################################/**
 # @RdocDefault callHooks
 #
-# @title "Call hook functions"
+# @title "Call hook functions by hook name"
 #
 # \description{
 #   @get "title".
@@ -11,17 +11,13 @@
 #
 # \arguments{
 #  \item{hookName}{A @character string of the hook name.}
-#  \item{...}{Argument passed to the hook function.}
+#  \item{...}{Argument passed to each hook function.}
 #  \item{removeCalledHooks}{If @TRUE, called hook functions are removed, 
 #     otherwise not.}
 # }
 #
 # \value{
-#   Returns (invisibly) a @list that is possible named with hook names,
-#   if possible.  Each element in the list is in turn a @list with three
-#   element: \code{fcn} is the hook function called, \code{result} 
-#   is its return value, and \code{exception} is the exception caught 
-#   or @NULL.
+#   Returns (invisibly) whatever @see "callHooks.list" returns.
 # }
 #
 # @examples "../incl/callHooks.Rex"
@@ -29,16 +25,20 @@
 # @author
 #
 # \seealso{
+#   Internally, after retriving hook functions, @see "callHooks.list" is
+#   called.
 #   See \link[base:UserHooks]{UserHooks} how to set hooks.
 # }
 #
 # @keyword programming
 #*/###########################################################################
 setMethodS3("callHooks", "default", function(hookName, ..., removeCalledHooks=FALSE) {
-  # Argument 'hook':
+  # Argument 'hookName':
   hookName <- as.character(hookName);
-  if (length(hookName) != 1)
-    throw("Argument 'hookName' must be a single character string: ", length(hookName));
+  if (length(hookName) != 1) {
+    throw("Argument 'hookName' must be a single character string: ", 
+                                                            length(hookName));
+  }
 
   # Argument 'removeCalledHooks':
   removeCalledHooks <- as.logical(removeCalledHooks);
@@ -50,15 +50,64 @@ setMethodS3("callHooks", "default", function(hookName, ..., removeCalledHooks=FA
   if (!is.list(hooks))
     hooks <- list(hooks);
 
-  toBeDone <- rep(TRUE, length=length(hooks));
   if (removeCalledHooks) {
-    on.exit(setHook(hookName, hooks[toBeDone], action="replace"));
+    on.exit(setHook(hookName, hooks[failedHooks], action="replace"));
   }
 
-  res <- vector(length(hooks), mode="list");
-  hookNames <- character(length(hooks));
-  for (kk in seq(length=length(hooks))) {
-    # Find the hook function
+  res <- callHooks(hooks, ...);
+  failedHooks <- attr(res, "failedHooks");
+
+  invisible(res);
+})
+
+
+
+###########################################################################/**
+# @class "function"
+# @RdocMethod callHooks
+# @alias callHooks.list
+#
+# @title "Call hook functions"
+#
+# \description{
+#   @get "title".
+# }
+#
+# @synopsis
+#
+# \arguments{
+#  \item{hooks}{A @function or a @list of hook @functions or names of such.}
+#  \item{...}{Argument passed to each hook function.}
+# }
+#
+# \value{
+#   Returns (invisibly) a @list that is named with hook names, if possible.  
+#   Each element in the list is in turn a @list with three element: 
+#   \code{fcn} is the hook function called, \code{result} is its return 
+#   value, and \code{exception} is the exception caught or @NULL.
+# }
+#
+# @author
+#
+# \seealso{
+#   See @see "callHooks" to call hook function by name.
+#   See \link[base:UserHooks]{UserHooks} how to set hooks.
+# }
+#
+# @keyword programming
+#*/###########################################################################
+setMethodS3("callHooks", "list", function(hooks, ...) {
+  # Argument 'hooks':
+  nhooks <- length(hooks);
+  if (nhooks == 0)
+    return();
+
+  res <- vector(nhooks, mode="list");
+
+  failedHooks <- rep(TRUE, length=nhooks);
+  hookNames <- character(nhooks);
+  for (kk in seq(length=nhooks)) {
+    # Get the hook function
     fcn <- hooks[[kk]];
     tmp <- list(fcn=fcn, result=NULL, exception=NULL);
     if (is.character(fcn)) {
@@ -76,7 +125,7 @@ setMethodS3("callHooks", "default", function(hookName, ..., removeCalledHooks=FA
       tryCatch({
         result <- fcn(...);
         tmp[["result"]] <- result;
-        toBeDone[kk] <- FALSE;
+        failedHooks[kk] <- FALSE;
       }, error = function(ex) {
         tmp[["exception"]] <<- ex;
       })
@@ -85,12 +134,22 @@ setMethodS3("callHooks", "default", function(hookName, ..., removeCalledHooks=FA
   }
 
   names(res) <- hookNames;
+  attr(res, "failedHooks") <- failedHooks;
+
   invisible(res);
 })
 
 
+setMethodS3("callHooks", "function", function(hooks, ...) {
+  callHooks(list(hooks), ...);
+})
+
+
 ############################################################################
 # HISTORY:
+# 2006-04-13
+# o Added callHooks() for lists.  This way hook functions can be pre-fetched
+#   by name once if called multiple times.
 # 2005-06-15
 # o Now callHooks() returns a detailed list of hooks called, their return
 #   values and any exceptions caught.
